package com.docus.server.message.service.impl;

import cn.hutool.http.HttpUtil;
import com.docus.core.util.Func;
import com.docus.infrastructure.web.api.ResultCode;
import com.docus.server.message.config.WsMessageForwardConfig;
import com.docus.server.message.consts.InteractiveMethod;
import com.docus.server.message.dto.Message;
import com.docus.server.message.dto.MessageResponse;
import com.docus.server.message.enums.DataFormatEnum;
import com.docus.server.message.result.MessageInfo;
import com.docus.server.message.result.ResultUtil;
import com.docus.server.message.service.UnifyMessageService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author WYBDEV
 */
@Service(InteractiveMethod.WS)
@Slf4j
public class WebServiceUnifyMessageServiceImpl implements UnifyMessageService {
    @Override
    public String pushInformation(String method, String message) {
        Message messageObj = new Message(method, message);
        WsMessageForwardConfig.WsMessageForward wsMessageForward = WsMessageForwardConfig.wsMessageForward(method);
        if (wsMessageForward == null || Func.isEmpty(wsMessageForward.getUrls())) {
            return "方法匹配错误！";
        }
        DataFormatEnum dataFormatEnum = DataFormatEnum.fromValue(wsMessageForward.getMessageDataType());
        if (dataFormatEnum == null) {
            return "格式匹配错误！";
        }
        try {
            List<MessageResponse> responseLis = new ArrayList<>();
            for (String url : wsMessageForward.getUrls()) {
                String resp = HttpUtil.post(url, Func.toJson(messageObj));
                responseLis.add(Func.readJson(resp, MessageResponse.class));
            }
            for (MessageResponse response : responseLis) {
                if (ResultCode.FAILED.getCode().equals(response.getRespCode())) {
                    return response.getRespMessage();
                }
            }
            responseLis = responseLis.stream().filter(item -> Func.isNotEmpty(item.getRespCode())).collect(Collectors.toList());
            if (Func.isEmpty(responseLis)) {
                throw new RuntimeException("未收到成功和失败的返回结果！");
            }
            return responseLis.get(0).getRespMessage();
        } catch (Exception ex) {
//            save(method,message);
            log.error(ex.getMessage(), ex);
            MessageInfo messageInfo = new MessageInfo(dataFormatEnum, message);
            return ResultUtil.dynamicParameterReplace(messageInfo, wsMessageForward.getErrorResult());
        }
    }
}
